/*
 * (c) 2013 Ch. klippel  -  ck@atelier-klippel.de / c.klippel@gmx.de
 *
 * This code is released under the terms of the GPL, version 2 or newer.
 * See the file COPYING for more information about the license.
 *
 * Changelog:
 *
 * v00.01: - Initial version
 *
 */

#include "rfsupply.h"

unsigned short adc_results_0[16], adc_results_1[16], adc_results_2[16];
unsigned char adc_idx;
unsigned char adc_chn;
ADREStype adc;

unsigned short adc_average_0, adc_average_1, adc_average_2;

volatile unsigned char adc_access_mutex;

static float adc_cal;
static float reg_cal;

RFDataType rf_data;

void adc_init(void)
{
    uart_print_start_msg(tr(S_ADC));

    ADCON1	= 0b00001100;
    ADCON0	= 0b00000001;
    ADCON2	= 0b10011011;
    PIE1bits.ADIE = 0;

    adc_access_mutex = 0;

    for(adc_chn = 0; adc_chn < 16; adc_chn++)
    {
        adc_results_0[adc_chn] = 0;
        adc_results_1[adc_chn] = 0;
        adc_results_2[adc_chn] = 0;
    }

    adc_average_0 = 0;
    adc_average_1 = 0;
    adc_average_2 = 0;

    adc_idx = 0;

    adc_chn = 0;
    adc_cal = ADC_CAL_VAL;
    adc_cal /= ADC_VCAL_VAL;

    reg_cal = 2.85; // 5.70

    rf_data.vf = 0;
    rf_data.vr = 0;
    rf_data.swr = 0;
    rf_data.pwr = 0;

    IPR1bits.TMR2IP = 0;
    PIE1bits.TMR2IE = 1;

    /*
    IPR1bits.ADIP = 0;
    PIE1bits.ADIE = 1;
    */
    ADCON0bits.GO = 1;

}

// the code for adc_process is done in interrupt.c
/*
void adc_process(void)
{
	if(ADCON0bits.GO == 0)
	{
            if(!(adc_mutex & 0x01))
            {
		adc.lo = ADRESL;
		adc.hi = ADRESH;

                if(adc_chn == 0)
		{
			ADCON0 = 0b00000101;
                        adc_average_0 -= adc_results_0[adc_idx];
        		adc_results_0[adc_idx] = adc.result;
                        adc_average_0 += adc.result;
                        adc_chn = 1;
		}
		else if (adc_chn == 1)
		{
			ADCON0 = 0b00001001;
                        adc_average_1 -= adc_results_1[adc_idx];
        		adc_results_1[adc_idx] = adc.result;
                        adc_average_1 += adc.result;
                        adc_chn = 2;
		}
		else if (adc_chn == 2)
		{
			ADCON0 = 0b00000001;
                        adc_average_2 -= adc_results_2[adc_idx];
        		adc_results_2[adc_idx] = adc.result;
                        adc_average_2 += adc.result;
                        adc_chn = 0;
                        adc_idx++;
                        adc_idx &= 0x0F;
		}
		
            }
            
            ADCON0bits.GO = 1;
	}
}
*/

unsigned short adc_get_reading(unsigned char which)
{
	static unsigned short res;
	static float val;

        val = 0.0;
        
        adc_access_mutex |= ADC_MUTEX_RESULTS;

        if(which == FORWARD)
        {
            val = adc_average_0;
        }
        else if(which == REFLECTED)
        {
            val = adc_average_1;
        }
        else if(which == SUPPLY)
        {
            val = adc_average_2;
            val *= reg_cal;
            val *= adc_cal;
        }
        
        adc_access_mutex &= ~ADC_MUTEX_RESULTS;

        res = val;
	return res;
}

void calc_rf_data(void)
{
    static float vf, vr;
    static float e1, e2, e3;

    adc_access_mutex |= ADC_MUTEX_RESULTS;

    rf_data.supply = adc_average_2;
    rf_data.vf = adc_average_0;
    rf_data.vr = adc_average_1;

    adc_access_mutex &= ~ADC_MUTEX_RESULTS;

    e1 = rf_data.supply;
    e1 *= reg_cal;
    e1 *= adc_cal;

    rf_data.supply = e1;

    rf_data.vf >>= 2;
    rf_data.vr >>= 2;

    vf = rf_data.vf;
    vr = rf_data.vr;

    if(vr == 0)
    {
        vr = 0.001;
    }

    if(vf > 0)
    {
        e1 = vr / vf;
        e2 = 1.0 + sqrt(e1);
        e3 = 1.0 - sqrt(e1);
        if(e3 > 0.0)
        {
            e1 = e2 / e3;
            e1 *= 100.0;
            rf_data.swr = e1;
/*
            rf_data.swr = e2 / e3;
            rf_data.swr *= 100.0;
*/
            e2 = vf + vr;
            e3 = vf - vr;

            e1 = 1.0 / e2;
            e1 *= e3;
            rf_data.pwr = e1;
/*
            rf_data.pwr = 1.0 / e2;
            rf_data.pwr *= e3;
*/
        }
        else
        {
            rf_data.swr = 9900;
            rf_data.vf = 0;
            rf_data.vr = 0;
            rf_data.pwr = 0;
        }
    }
    else
    {
        rf_data.swr = 8800;
        rf_data.vf = 0;
        rf_data.vr = 0;
        rf_data.pwr = 0;
    }
}

