/*
 * (c) 2013 Ch. klippel  -  ck@atelier-klippel.de / c.klippel@gmx.de
 *
 * This code is released under the terms of the GPL, version 2 or newer.
 * See the file COPYING for more information about the license.
 *
 * Changelog:
 *
 * v00.01: - Initial version
 *
 */

#include "rfsupply.h"

ConfigDataType config;
MainConfigDataType main_config;

unsigned char editable[64];
static unsigned short cfg_mem_crc;

void config_memory_init(void)
{
    uart_print_start_msg(tr(S_CONFIG_MEMORY));
}

void config_memory_default_main_config(void)
{
    uart_print_module_msg(tr(S_CONFIG_MEMORY), tr(S_USING_INTMAINCFG));

    main_config.language            = LANG_ENGLISH;
    main_config.startup_config      = STARTUP_OFF;
    main_config.config_num          = 0;
    main_config.version_major       = VERSION_MAJOR;
    main_config.version_minor       = VERSION_MINOR;

    main_config.crc = crc_calc_block(MAIN_CONFIG_SEED, MAIN_CONFIG_PAYLOAD_SIZE, main_config.data);

    if(languages_get_lang() != main_config.language)
    {
        languages_set_lang(main_config.language);
    }
}

unsigned char config_memory_get_language(void)
{
    eeprom_read_block(0, MAIN_CONFIG_SIZE, (unsigned char*) &main_config.raw);

    if(main_config.crc != crc_calc_block(MAIN_CONFIG_SEED, MAIN_CONFIG_PAYLOAD_SIZE, main_config.data))
    {
        return 0;
    }
    else
    {
        return main_config.language;
    }
}

unsigned char config_memory_load_main_config(void)
{
    uart_print_module_msg(tr(S_CONFIG_MEMORY), tr(S_USING_SAVEDMAINCFG));
    
    eeprom_read_block(0, MAIN_CONFIG_SIZE, (unsigned char*) &main_config.raw);

    if(main_config.crc != crc_calc_block(MAIN_CONFIG_SEED, MAIN_CONFIG_PAYLOAD_SIZE, main_config.data))
    {
        uart_print_module_msg(tr(S_CONFIG_MEMORY), tr(S_CHECKSUM_WRONG));
        return 0;
    }
    else
    {
        uart_print_module_msg(tr(S_CONFIG_MEMORY), tr(S_CHECKSUM_OK));
        return 1;
    }
}

void config_memory_save_main_config(void)
{
    uart_print_module_msg(tr(S_CONFIG_MEMORY), tr(S_SAVING_MAINCFG));

    main_config.crc = crc_calc_block(MAIN_CONFIG_SEED, MAIN_CONFIG_PAYLOAD_SIZE, main_config.data);
    eeprom_write_block(0, MAIN_CONFIG_SIZE, (unsigned char*) &main_config.raw);
}

void config_memory_default_config(void)
{
    uart_print_module_msg(tr(S_CONFIG_MEMORY), tr(S_USING_INTCFG));

    copy_romstring_to_ram("Default Config  ", config.name, 16);

    config.pm_config                = PM_CFG_ENABLE_SLEEP | PM_CFG_ENABLE_POWERDOWN;
    config.pm_manual_default        = PM_MANUAL_NORMAL;
    config.pm_startup_config        = PM_CFG_STARTUP_NORMAL;

    config.pm_wakeup_config         = (PM_CFG_WAKEUP_BOOST | PM_CFG_WAKEUP_TIMED);

    config.startup_limit_swr        = 400;
    config.startup_limit_delta      = 5;
    config.startup_limit_time       = 5;

    config.startup_boost_swr        = 400;
    config.startup_boost_delta      = 5;
    config.startup_boost_time       = 5;

    config.wakeup_boost_swr         = 400;
    config.wakeup_boost_delta       = 5;
    config.wakeup_boost_time        = 5;

    config.enter_idle_time          = 10;
    config.idle_threshold           = 30;
    config.sleep_start_threshold    = 70;
    config.wakeup_threshold         = 40;

    config.sleep_time               = 30;
    config.off_time                 = 3600;
    config.enter_sleep_time         = 5;
    config.light_sleep_time         = 5;

    config.normal_backlight         = BACKLIGHT_FADE;
    config.sleep_backlight          = BACKLIGHT_PULSE;
    config.auto_off_backlight       = BACKLIGHT_FLASH;

    config.backlight_default        = 63;

    config.low_pulse_backlight      = 15;
    config.hi_pulse_backlight       = 40;
    config.backlight_pulse_speed    = 250;

    config.low_flash_backlight      = 5;
    config.hi_flash_backlight       = 15;
    config.backlight_flash_ton      = 100;
    config.backlight_flash_toff     = 5000;

    config.crc = crc_calc_block(CONFIG_SEED, CONFIG_PAYLOAD_SIZE, config.data);
}

unsigned char config_memory_load_config(unsigned char cfg_num)
{
    uart_print_module_msg(tr(S_CONFIG_MEMORY), tr(S_USING_SAVEDCFG));

    eeprom_read_block(MAIN_CONFIG_SIZE + (CONFIG_SIZE * cfg_num), CONFIG_SIZE, (unsigned char*) &config.raw);

    if(config.crc != crc_calc_block(CONFIG_SEED, CONFIG_PAYLOAD_SIZE, config.data))
    {
        uart_print_module_msg(tr(S_CONFIG_MEMORY), tr(S_CHECKSUM_WRONG));
        return 0;
    }
    else
    {
        uart_print_module_msg(tr(S_CONFIG_MEMORY), tr(S_CHECKSUM_OK));
        uart_print_module_msg_ramval(tr(S_CONFIG_MEMORY), tr(S_LOADED_CFG), (char*) &config.name);
        return 1;
    }
}

void config_memory_save_config(unsigned char cfg_num)
{
    uart_print_module_msg_ramval(tr(S_CONFIG_MEMORY), tr(S_SAVING_CFG), (char*) &config.name);

    config.crc = crc_calc_block(CONFIG_SEED, CONFIG_PAYLOAD_SIZE, config.data);
    eeprom_write_block(MAIN_CONFIG_SIZE + (CONFIG_SIZE * cfg_num), CONFIG_SIZE, (unsigned char*) &config.raw);
}

