/*
 * (c) 2013 Ch. klippel  -  ck@atelier-klippel.de / c.klippel@gmx.de
 *
 * This code is released under the terms of the GPL, version 2 or newer.
 * See the file COPYING for more information about the license.
 *
 * Changelog:
 *
 * v00.01: - Initial version
 *
 */

#ifndef CONFIG_MEMORY_H
#define CONFIG_MEMORY_H

#include "version.h"

typedef struct ConfigDataStructDef
{
    char            name[16];                   //  16
    unsigned char   pm_config;                  //   1
    unsigned char   pm_manual_default;          //   1
    unsigned char   pm_startup_config;          //   1
    unsigned char   pm_wakeup_config;           //   1
    unsigned short  startup_limit_swr;          //   2
    unsigned char   startup_limit_delta;        //   1
    unsigned char   startup_limit_time;         //   1
    unsigned short  startup_boost_swr;          //   2
    unsigned char   startup_boost_delta;        //   1
    unsigned char   startup_boost_time;         //   1
    unsigned short  wakeup_boost_swr;           //   2
    unsigned char   wakeup_boost_delta;         //   1
    unsigned char   wakeup_boost_time;          //   1
    unsigned char   enter_idle_time;            //   1
    unsigned char   idle_threshold;             //   1
    unsigned char   sleep_start_threshold;      //   1
    unsigned char   wakeup_threshold;           //   1
    unsigned short  sleep_time;                 //   2
    unsigned short  off_time;                   //   2
    unsigned char   enter_sleep_time;           //   1
    unsigned char   light_sleep_time;           //   1

    unsigned char   normal_backlight;           //   1
    unsigned char   sleep_backlight;            //   1
    unsigned char   auto_off_backlight;         //   1

    unsigned char   backlight_default;          //   1

    unsigned char   low_pulse_backlight;        //   1
    unsigned char   hi_pulse_backlight;         //   1
    unsigned short  backlight_pulse_speed;      //   2

    unsigned char   low_flash_backlight;        //   1
    unsigned char   hi_flash_backlight;         //   1
    unsigned short  backlight_flash_ton;        //   2
    unsigned short  backlight_flash_toff;       //   2

} ConfigDataStruct;

typedef union ConfigDataTypeDef
{
    unsigned char       raw[sizeof(ConfigDataStruct)+2];
    struct
    {
        unsigned char   data[sizeof(ConfigDataStruct)];
        unsigned short  crc;
    };
    struct              ConfigDataStructDef;
} ConfigDataType;

typedef struct MainConfigDataStructDef
{
    unsigned char   version_major;
    unsigned char   version_minor;
    unsigned char   language;
    unsigned char   startup_config;
    unsigned char   config_num;

} MainConfigDataStruct;

typedef union MainConfigDataTypeDef
{
    unsigned char       raw[sizeof(MainConfigDataStruct)+2];
    struct
    {
        unsigned char   data[sizeof(MainConfigDataStruct)];
        unsigned short  crc;
    };
    struct              MainConfigDataStructDef;
} MainConfigDataType;

#define MAIN_CONFIG_PAYLOAD_SIZE    sizeof(MainConfigDataStruct)
#define CONFIG_PAYLOAD_SIZE         sizeof(ConfigDataStruct)

#define MAIN_CONFIG_SIZE            sizeof(MainConfigDataType)
#define CONFIG_SIZE                 sizeof(ConfigDataType)

#define MAIN_CONFIG_START       0

#define CONFIG_1_START          ( 0 + MAIN_CONFIG_SIZE )
#define CONFIG_2_START          ( CONFIG_1_START + CONFIG_SIZE )
#define CONFIG_3_START          ( CONFIG_2_START + CONFIG_SIZE )
#define CONFIG_4_START          ( CONFIG_3_START + CONFIG_SIZE )
#define CONFIG_5_START          ( CONFIG_4_START + CONFIG_SIZE )
#define CONFIG_6_START          ( CONFIG_5_START + CONFIG_SIZE )
#define CONFIG_7_START          ( CONFIG_6_START + CONFIG_SIZE )
#define CONFIG_8_START          ( CONFIG_7_START + CONFIG_SIZE )

#define CONFIG_END              ( CONFIG_8_START + CONFIG_SIZE )

extern ConfigDataType config;
extern MainConfigDataType main_config;

extern unsigned char editable[64];

void config_memory_init(void);

unsigned char config_memory_get_language(void);

void config_memory_default_main_config(void);
unsigned char config_memory_load_main_config(void);
void config_memory_save_main_config(void);

void config_memory_default_config(void);
unsigned char config_memory_load_config(unsigned char cfg_num);
void config_memory_save_config(unsigned char cfg_num);

#endif

