/*
 * (c) 2013 Ch. klippel  -  ck@atelier-klippel.de / c.klippel@gmx.de
 *
 * This code is released under the terms of the GPL, version 2 or newer.
 * See the file COPYING for more information about the license.
 *
 * Changelog:
 *
 * v00.01: - Initial version
 *
 */

#include "rfsupply.h"

static unsigned short num;

void lcd_delay_short(void)
{
	static unsigned short cnt;

	for(cnt = 0; cnt < 100; cnt ++) // 250
	{
		_asm NOP _endasm;
	}
}

void lcd_delay_long(void)
{
	static unsigned short cnt;

	for(cnt = 0; cnt < 2500; cnt ++) // 2500
	{
		_asm NOP _endasm;
	}
}

void lcd_write_data_single(unsigned char dta)
{
	LCD_RS = LCD_DATA;
	LCD_RW = LCD_WRITE;
        SET_LCD_DIR_OUT();

      	lcd_delay_short();

        LCD_DATA_OUT = (dta>>4);

	LCD_ENABLE = LCD_ENABLE_HI;
	lcd_delay_short();
	LCD_ENABLE = LCD_ENABLE_LO;

	LCD_DATA_OUT = (dta & 0x0F);

        LCD_ENABLE = LCD_ENABLE_HI;
	lcd_delay_short();
	LCD_ENABLE = LCD_ENABLE_LO;

        lcd_delay_short();
}	

void lcd_write_command_single(unsigned char cmd)
{
	LCD_RS = LCD_CMD;
	LCD_RW = LCD_WRITE;
        SET_LCD_DIR_OUT();

      	lcd_delay_short();

	LCD_DATA_OUT = (cmd >> 4);

        LCD_ENABLE = LCD_ENABLE_HI;
	lcd_delay_short();
	LCD_ENABLE = LCD_ENABLE_LO;

        LCD_DATA_OUT = (cmd & 0x0F);

	LCD_ENABLE = LCD_ENABLE_HI;
	lcd_delay_short();
	LCD_ENABLE = LCD_ENABLE_LO;

        lcd_delay_short();
}	


void lcd_init(void)
{
	static unsigned long cnt1;
	static unsigned char cnt2;

        uart_print_start_msg(tr(S_LCD_DRIVER));

        cnt1 = 100000;
	while(cnt1--) { _asm NOP _endasm; }

        // init display in 4-bit mode
        lcd_write_command_single(0x33);
        lcd_write_command_single(0x32);
        
	lcd_write_command_single(LCD_FUNC_SET | LCD_FUNC_4BIT | LCD_FUNC_5x8 | LCD_FUNC_2LINE);
      	lcd_delay_long();

	lcd_write_command_single(LCD_CTRL | LCD_CTRL_DISP_OFF);
	lcd_delay_long();

	lcd_write_command_single(LCD_CLEAR);
	lcd_delay_long();

	lcd_write_command_single(LCD_MODE | LCD_MODE_INCREMENT | LCD_MODE_NOSHIFT);
	lcd_delay_long();

	lcd_write_command_single(LCD_CTRL | LCD_CTRL_DISP_ON | LCD_CTRL_CURSOR_OFF | LCD_CTRL_CURSOR_NOBLINK);
	lcd_delay_long();
	
	CCP2CON = 0b00001100;
	PR2 = 0xFF;
	TMR2 = 0;
	T2CON = 0b01111000;
	CCPR2L = 0x00;
	T2CONbits.TMR2ON = 1;
	
	lcd_write_string("----------------");
		
	for(cnt2 = 0x10; cnt2 < 0x18; cnt2++)
	{
		lcd_write_command_single(LCD_SET_CGRAM | cnt2);
		lcd_write_data_single(0xFF);
	}

        lcd_write_string_at(0, 0, "                ");

}

void lcd_write_string(const rom char *str)
{
	do
	{
		lcd_write_data_single(*str++);
	} while(*str);
}

void lcd_write_string_at(unsigned char col, unsigned char row, const rom char *str)
{
	static unsigned char adr;
	
	if(col >= LCD_CHARS_PER_LINE || row >= LCD_NUM_LINES)
	{
		return;
	}

	adr = col;
	if(row)
	{
		adr |= 0x40;
	}

	lcd_write_command_single(LCD_SET_DDRAM | adr);
	
	do
	{
		lcd_write_data_single(*str++);
	} while(*str);
}

void lcd_write_char_at(unsigned char col, unsigned char row, unsigned char chr)
{
	static unsigned char adr;

	if(col >= LCD_CHARS_PER_LINE || row >= LCD_NUM_LINES)
	{
		return;
	}

	adr = col;
	if(row)
	{
		adr |= 0x40;
	}

	lcd_write_command_single(LCD_SET_DDRAM | adr);

	lcd_write_data_single(chr);
}

void lcd_draw_bargraph(unsigned char col, unsigned char row, unsigned char val)
{
	static unsigned char pos, uchar;
	static unsigned char remainder;

      	if(col >= LCD_CHARS_PER_LINE || row >= LCD_NUM_LINES)
	{
		return;
	}

	pos = col;
	if(row)
	{
		pos |= 0x40;
	}

	lcd_write_command_single(LCD_SET_DDRAM | pos);
/*
	if(row)
	{
		pos = 0x40;
		uchar = 0x08;
	}
	else
	{
		pos = 0x00;
		uchar = 0x00;
	}
	
	lcd_write_command_single(LCD_SET_DDRAM | pos);
*/
  	remainder = val;
	
	while(remainder >= 5)
	{
		remainder -= 5;		
		lcd_write_data_single(0x02);
		pos++;
	}

	if(remainder)
	{
		uchar = 0x1F;
		uchar <<= (5-remainder);


                if(pos & 0x40)
		{
			lcd_write_command_single(LCD_SET_CGRAM | 0x08);
		}
		else
		{
			lcd_write_command_single(LCD_SET_CGRAM | 0x00);
		}

		for(remainder = 0; remainder < 8; remainder++)
		{
			lcd_write_data_single(uchar);
		}

                lcd_write_command_single(LCD_SET_DDRAM | pos);

                if(pos & 0x40)
		{
			lcd_write_data_single(0x01);
		}
		else
		{
			lcd_write_data_single(0x00);
		}

                pos++;
	}

        pos &= 0x0F;

        while(pos < LCD_CHARS_PER_LINE)
        {
                lcd_write_data_single(' ');
                pos++;
        }
}

void lcd_write_num(unsigned char col, unsigned char row, signed short val)
{
	static unsigned char adr;
	
	if(col >= LCD_CHARS_PER_LINE || row >= LCD_NUM_LINES)
	{
		return;
	}

	adr = col;
	
	if(row)
	{
		adr |= 0x40;
	}

	lcd_write_command_single(LCD_SET_DDRAM | adr);
	
	if(val < 0)
	{
		val = -val;
		lcd_write_data_single('-');
	}
	else
	{
		lcd_write_data_single(' ');
	}		

        divmod_set(val, 1000);
	lcd_write_data_single('0'+divmod());

        divmod_set_divider(100);
	lcd_write_data_single('0'+divmod());

        divmod_set_divider(10);
	lcd_write_data_single('0'+divmod());

	lcd_write_data_single('0'+divmod_get_remainder());
}

void lcd_write_dnum(unsigned char col, unsigned char row, signed short val)
{
	static unsigned char adr, res;
		
	if(col >= LCD_CHARS_PER_LINE || row >= LCD_NUM_LINES)
	{
		return;
	}

	if(val > 9999)
	{
		lcd_write_string_at(col, row, "*OVR*");
                return;
	}
	
	adr = col;
	
	if(row)
	{
		adr |= 0x40;
	}

	lcd_write_command_single(LCD_SET_DDRAM | adr);

        divmod_set(val, 1000);

        res = divmod();
	
	if(res > 0)
	{
		lcd_write_data_single('0'+res);
	}
	else
	{
		lcd_write_data_single(' ');
	}
	
	divmod_set_divider(100);
	lcd_write_data_single('0'+divmod());

	lcd_write_data_single('.');

        divmod_set_divider(10);
        lcd_write_data_single('0'+divmod());

	lcd_write_data_single('0'+divmod_get_remainder());
}

