/*
 * (c) 2013 Ch. klippel  -  ck@atelier-klippel.de / c.klippel@gmx.de
 *
 * This code is released under the terms of the GPL, version 2 or newer.
 * See the file COPYING for more information about the license.
 *
 * Changelog:
 *
 * v00.01: - Initial version
 *
 */

#include "rfsupply.h"

static unsigned long key_cnt;
static ADREStype test;

void main(void)
{
    static signed char updown;
    static unsigned char bar_val, old_bar_val;
    static unsigned char cur_pm_state, old_pm_state;
    static unsigned char cur_pwr, old_pwr;
    static unsigned char cur_tip, old_tip;
    static unsigned char show_info;

    // the following modules must be initialized first in the given order
    hardware_init();
    interrupt_init();
    uart_init();
    languages_init();
    config_memory_init();
    config_manager_init();

    // these modules can be initialized in any order
    lcd_init();
    backlight_init();
    encoder_init();
    tip_detect_init();
    adc_init();
    rf_manager_init();
    power_manager_init();

    encoder_reset_button();

    updown = encoder_get();
    updown = 0;
    bar_val = 0;
    old_bar_val = 0;

    cur_pm_state = power_manager_get_state();
    old_pm_state = 0xFF;

    cur_pwr = 0;
    old_pwr = 0xFF;

    cur_tip = 0;
    old_tip = 0xFF;
    
    show_info = 0;

    backlight_mode(config.normal_backlight);

    while(1)
    {
        if(ENC1_K == 0)
        {
            key_cnt++;
            if(key_cnt >= 100)
            { 
                uart_print_start_msg("***** RESET *****\r\n");
                while(ENC1_K == 0)
                {
                    _asm NOP _endasm;
                }
                _asm RESET _endasm
            }
        }
        else
        {
            key_cnt = 0;
        }
        updown = encoder_get();
        power_manager_process(updown);
        
        cur_pm_state = power_manager_get_state();
        cur_pwr = rf_manager_get_mode();

        if((cur_pm_state != PM_OFF) && (cur_pm_state != PM_MANUAL_OFF) && (cur_pm_state != PM_POWERDOWN))
        {
            cur_tip = tip_detect_get_status();
            if(old_tip != cur_tip)
            {
                if(cur_tip == TIP_MISSING)
                {
                    old_pwr = 0xFF;
                    old_pm_state = 0xFF;
                    lcd_write_string_at(0, 0, "!! ATTENTION !! ");
                    lcd_write_string_at(0, 1, "No tip detected ");
                }
                
                old_tip = cur_tip;
            }
        }
        else if((cur_pm_state == PM_OFF) || (cur_pm_state == PM_MANUAL_OFF))
        {
            old_pwr = 0xFF;
            lcd_write_string_at(0, 0, "----- OFF ----- ");
            lcd_write_string_at(0, 1, "=============== ");
        }

        if(cur_tip == TIP_OK)
        {
            if(old_pm_state != cur_pm_state)
            {
                old_pm_state = cur_pm_state;

                if(cur_pm_state == PM_SLEEP)
                {
                    lcd_write_string_at(0, 0, "<<< Sleeping >>>");
                    lcd_write_string_at(0, 1, "================");
                    show_info = 0;
                }
                else if(cur_pm_state == PM_POWERDOWN)
                {
                    lcd_write_string_at(0, 0, "** Powerdown ** ");
                    lcd_write_string_at(0, 1, "=============== ");
                    show_info = 0;
                }
                else if((cur_pm_state == PM_OFF) || (cur_pm_state == PM_MANUAL_OFF))
                {
                    lcd_write_string_at(0, 0, "----- OFF ----- ");
                    lcd_write_string_at(0, 1, "=============== ");
                    show_info = 0;
                }
                else
                {
                    show_info = 1;
                    switch(cur_pm_state)
                    {
                        case PM_HEATING:
                                        lcd_write_string_at(0, 1, tr(S_HEATING));
                                        break;

                        case PM_HEATED:
                                        lcd_write_string_at(0, 1, tr(S_READY));
                                        break;

                        case PM_IDLE:
                                        lcd_write_string_at(0, 1, tr(S_IDLE));
                                        break;

                        case PM_ENTER_SLEEP:
                                        lcd_write_string_at(0, 1, tr(S_ENTER_SLEEP));
                                        break;

                        case PM_WAKEUP:
                                        lcd_write_string_at(0, 1, tr(S_WAKEUP));
                                        break;

                        case PM_MANUAL_OFF:
                                        lcd_write_string_at(0, 1, tr(S_MPOWER_OFF));
                                        break;

                        case PM_MANUAL_LIMIT:
                                        lcd_write_string_at(0, 1, tr(S_MPOWER_LIMIT));
                                        break;

                        case PM_MANUAL_NORMAL:
                                        lcd_write_string_at(0, 1, tr(S_MPOWER_NORMAL));
                                        break;

                        case PM_MANUAL_BOOST:
                                        lcd_write_string_at(0, 1, tr(S_MPOWER_BOOST));
                                        break;

                    }
                }
            }

            if(show_info)
            {
                if(old_pwr != cur_pwr)
                {
                    old_pwr = cur_pwr;
                    lcd_write_string_at(1, 0, " ");
                    switch(cur_pwr)
                    {
                        case RF_POWER_OFF:
                                        lcd_write_string_at(0, 0, "O");
                                        break;

                        case RF_POWER_LIMIT:
                                        lcd_write_string_at(0, 0, "L");
                                        break;

                        case RF_POWER_NORMAL:
                                        lcd_write_string_at(0, 0, "N");
                                        break;

                        case RF_POWER_BOOST:
                                        lcd_write_string_at(0, 0, "B");
                                        break;
                    }

                }

                bar_val = rf_manager_get_pwr(70);
                if(bar_val != old_bar_val)
                {
                    lcd_draw_bargraph(2, 0, bar_val);
                    old_bar_val = bar_val;
                }
            }
            lcd_write_dnum(11, 1, rf_manager_get_supply());
            lcd_write_num(0, 1, rf_manager_get_delta());
            lcd_write_num(5, 1, rf_manager_get_delta_diff());
            lcd_write_string_at(10, 1, " ");
        }
    }
}

