/*
 * (c) 2013 Ch. klippel  -  ck@atelier-klippel.de / c.klippel@gmx.de
 *
 * This code is released under the terms of the GPL, version 2 or newer.
 * See the file COPYING for more information about the license.
 *
 * Changelog:
 *
 * v00.01: - Initial version
 *
 */

#include "rfsupply.h"

static unsigned char pm_state, pm_old_state, pm_saved_state;
static unsigned char old_button, button_clicked;
static unsigned long pm_seconds;

//static unsigned char old_pwr_setting;

void power_manager_init(void)
{
    uart_print_start_msg(tr(S_POWER_MANAGER));

    power_manager_reset();

    pm_seconds = 0;

    old_button = 1;
    button_clicked = 0;
}

void power_manager_reset(void)
{
//    old_pwr_setting = RF_POWER_OFF;

    RFG_EN = 0;
    VLIM_DIR = PIN_IN;

    pm_old_state = PM_OFF;
    pm_state = PM_OFF;

    if(main_config.startup_config & STARTUP_ON)
    {
        power_manager_set_state(PM_STARTUP);
    }
    else if(main_config.startup_config & STARTUP_MANUAL_OFF)
    {
        power_manager_set_state(PM_MANUAL_OFF);
    }
    else if(main_config.startup_config & STARTUP_MANUAL_LIMIT)
    {
        power_manager_set_state(PM_MANUAL_LIMIT);
    }
    else if(main_config.startup_config & STARTUP_MANUAL_NORMAL)
    {
        power_manager_set_state(PM_MANUAL_NORMAL);
    }
    else if(main_config.startup_config & STARTUP_MANUAL_BOOST)
    {
        power_manager_set_state(PM_MANUAL_BOOST);
    }
    else
    {
        power_manager_set_state(PM_OFF);
    }

    pm_saved_state = config.pm_manual_default;
}

void power_manager_process(signed char user_input)
{
    if(ticker != 0)
    {
        rf_manager_store_delta();
        rf_manager_clear_delta();
        pm_seconds += ticker;
        ticker = 0;
    }

    if(old_button != ENC1_K)
    {
        if(old_button == 0)
        {
            button_clicked = 1;
        }
        old_button = ENC1_K;
    }
    
    switch(pm_state)
    {
        case PM_STARTUP:
            RFG_EN = 1;
            backlight_mode(config.normal_backlight);
            if(config.pm_startup_config & PM_CFG_STARTUP_LIMIT)
            {
                if(config.pm_startup_config & PM_CFG_STARTUP_TIMED)
                {
                    power_manager_set_state(PM_STARTUP_TIMED_LIMIT);
                }
                else if(config.pm_startup_config & PM_CFG_STARTUP_SWR)
                {
                    power_manager_set_state(PM_STARTUP_SWR_LIMIT);
                }
                else
                {
                    power_manager_set_state(PM_STARTUP_DELTA_LIMIT);
                }
                rf_manager_set_mode(RF_POWER_LIMIT);
            }
            else if(config.pm_startup_config & PM_CFG_STARTUP_BOOST)
            {
                if(config.pm_startup_config & PM_CFG_STARTUP_TIMED)
                {
                    power_manager_set_state(PM_STARTUP_TIMED_BOOST);
                }
                else if(config.pm_startup_config & PM_CFG_STARTUP_SWR)
                {
                    power_manager_set_state(PM_STARTUP_SWR_BOOST);
                }
                else
                {
                    power_manager_set_state(PM_STARTUP_DELTA_BOOST);
                }
                rf_manager_set_mode(RF_POWER_BOOST);
            }
            else
            {
                power_manager_set_state(PM_HEATING);
                rf_manager_set_mode(RF_POWER_NORMAL);
            }
            break;

        case PM_STARTUP_SWR_LIMIT:
            if(rf_manager_get_swr() >= config.startup_limit_swr)
            {
                power_manager_set_state(PM_HEATING);
            }
            break;

        case PM_STARTUP_TIMED_LIMIT:
            if(pm_seconds >= config.startup_limit_time)
            {
                power_manager_set_state(PM_HEATING);
            }
            break;

        case PM_STARTUP_DELTA_LIMIT:
            if(rf_manager_has_delta(config.startup_limit_delta) == 0)
            {
                power_manager_set_state(PM_HEATING);
            }
            break;

        case PM_STARTUP_SWR_BOOST:
            if(rf_manager_get_swr() >= config.startup_boost_swr)
            {
                power_manager_set_state(PM_HEATING);
            }
            break;

        case PM_STARTUP_TIMED_BOOST:
            if(pm_seconds >= config.startup_boost_time)
            {
                power_manager_set_state(PM_HEATING);
            }
            break;

        case PM_STARTUP_DELTA_BOOST:
            if(rf_manager_has_delta(config.startup_boost_delta) == 0)
            {
                power_manager_set_state(PM_HEATING);
            }
            break;

        case PM_HEATING:
            if(rf_manager_has_delta(config.idle_threshold) == 0)
            {
                power_manager_set_state(PM_HEATED);
            }
            break;

        case PM_HEATED:
            if(rf_manager_has_delta(config.idle_threshold) == 0)
            {
                power_manager_set_state(PM_ENTER_IDLE);
            }
            break;

        case PM_ENTER_IDLE:
            if(rf_manager_has_delta(config.idle_threshold))
            {
                power_manager_set_state(PM_HEATING);
            }
            else if(pm_seconds >= config.enter_idle_time)
            {
                power_manager_set_state(PM_IDLE);
            }
            break;

        case PM_IDLE:
            if(rf_manager_has_delta(config.idle_threshold))
            {
                power_manager_set_state(PM_HEATING);
            }
            else
            {
                if(config.pm_config & PM_CFG_ENABLE_SLEEP)
                {
                    if(pm_seconds >= config.sleep_time)
                    {
                        rf_manager_clear_all_delta();
                        power_manager_set_state(PM_ENTER_SLEEP);
                        rf_manager_set_mode(RF_POWER_LIMIT);
                    }
                }
            }
            break;
            
        case PM_ENTER_SLEEP:
            if(pm_seconds >= config.enter_sleep_time)
            {
                backlight_mode(config.sleep_backlight);
                power_manager_set_state(PM_SLEEP);
            }

            if((user_input != 0) || (button_clicked))
            {
                power_manager_set_state(PM_WAKEUP);
            }
            break;

        case PM_SLEEP:
            if(pm_seconds < config.light_sleep_time)
            {
                if(rf_manager_has_delta(config.sleep_start_threshold))
                {
                   power_manager_set_state(PM_WAKEUP);
                }
            }
            else if((rf_manager_has_delta(config.wakeup_threshold) != 0) || (user_input != 0) || (button_clicked != 0))
            {
               power_manager_set_state(PM_WAKEUP);
            }

            if(config.pm_config & PM_CFG_ENABLE_POWERDOWN)
            {
                if(pm_seconds >= config.off_time)
                {
                    power_manager_set_state(PM_ENTER_POWERDOWN);
                }
            }

            break;

        case PM_WAKEUP:
            backlight_mode(config.normal_backlight);
            if(config.pm_wakeup_config & PM_CFG_WAKEUP_BOOST)
            {
                if(config.pm_wakeup_config & PM_CFG_WAKEUP_TIMED)
                {
                    power_manager_set_state(PM_WAKEUP_TIMED_BOOST);
                }
                else if(config.pm_wakeup_config & PM_CFG_WAKEUP_SWR)
                {
                    power_manager_set_state(PM_WAKEUP_SWR_BOOST);
                }
                else
                {
                    power_manager_set_state(PM_WAKEUP_DELTA_BOOST);
                }
                rf_manager_set_mode(RF_POWER_BOOST);
            }
            else
            {
                power_manager_set_state(PM_HEATING);
                rf_manager_set_mode(RF_POWER_NORMAL);
            }
            break;

        case PM_WAKEUP_SWR_BOOST:
            if(rf_manager_get_swr() >= config.wakeup_boost_swr)
            {
                power_manager_set_state(PM_HEATING);
                rf_manager_set_mode(RF_POWER_NORMAL);
            }
            break;

        case PM_WAKEUP_TIMED_BOOST:
            if(pm_seconds >= config.wakeup_boost_time)
            {
                power_manager_set_state(PM_HEATING);
                rf_manager_set_mode(RF_POWER_NORMAL);
            }
            break;

        case PM_WAKEUP_DELTA_BOOST:
            if(rf_manager_has_delta(config.wakeup_boost_delta) == 0)
            {
                power_manager_set_state(PM_HEATING);
                rf_manager_set_mode(RF_POWER_NORMAL);
            }
            break;

        case PM_ENTER_POWERDOWN:
            backlight_mode(config.auto_off_backlight);
            power_manager_set_state(PM_POWERDOWN);
            rf_manager_set_mode(RF_POWER_OFF);
            break;

        case PM_POWERDOWN:
            if((user_input != 0) || (button_clicked))
            {
                power_manager_set_state(PM_STARTUP);
            }
            break;

        case PM_MANUAL_OFF:
            if(rf_manager_get_mode() != RF_POWER_OFF)
            {
                rf_manager_set_mode(RF_POWER_OFF);
            }
            if(button_clicked)
            {
                switch(pm_saved_state)
                {
                    case PM_MANUAL_LIMIT:
                        power_manager_set_state(PM_MANUAL_LIMIT);
                        rf_manager_set_mode(RF_POWER_LIMIT);
                        break;

                    case PM_MANUAL_NORMAL:
                        power_manager_set_state(PM_MANUAL_NORMAL);
                        rf_manager_set_mode(RF_POWER_NORMAL);
                        break;

                    case PM_MANUAL_BOOST:
                        power_manager_set_state(PM_MANUAL_BOOST);
                        rf_manager_set_mode(RF_POWER_BOOST);
                        break;
                }
            }

            if(user_input > 0)
            {
                power_manager_set_state(PM_MANUAL_LIMIT);
                rf_manager_set_mode(RF_POWER_LIMIT);
            }

            break;

        case PM_MANUAL_LIMIT:
            if(user_input > 0)
            {
                power_manager_set_state(PM_MANUAL_NORMAL);
                rf_manager_set_mode(RF_POWER_NORMAL);
            }

            if(button_clicked)
            {
                pm_saved_state = pm_state;
                power_manager_set_state(PM_MANUAL_OFF);
                rf_manager_set_mode(RF_POWER_OFF);
            }
            break;

        case PM_MANUAL_NORMAL:
            if(user_input < 0)
            {
                power_manager_set_state(PM_MANUAL_LIMIT);
                rf_manager_set_mode(RF_POWER_LIMIT);
            }
            else if(user_input > 0)
            {
                power_manager_set_state(PM_MANUAL_BOOST);
                rf_manager_set_mode(RF_POWER_BOOST);
            }

            if(button_clicked)
            {
                pm_saved_state = pm_state;
                power_manager_set_state(PM_MANUAL_OFF);
                rf_manager_set_mode(RF_POWER_OFF);
            }
            break;

        case PM_MANUAL_BOOST:
            if(user_input < 0)
            {
                power_manager_set_state(PM_MANUAL_NORMAL);
                rf_manager_set_mode(RF_POWER_NORMAL);
            }
            
            if(button_clicked)
            {
                pm_saved_state = pm_state;
                power_manager_set_state(PM_MANUAL_OFF);
                rf_manager_set_mode(RF_POWER_OFF);
            }
            break;

        case PM_OFF:
            if(rf_manager_get_mode() != RF_POWER_OFF)
            {
                rf_manager_set_mode(RF_POWER_OFF);
            }
            if((user_input != 0) || (button_clicked != 0))
            {
                power_manager_set_state(PM_STARTUP);
            }
            break;

        default:
            break;
    }
    
    button_clicked = 0;
}

void power_manager_set_state(unsigned char new_state)
{
    if(pm_old_state != new_state)
    {
        uart_print_timed_change_msg(tr(S_POWER_MANAGER), tr(S_CHANGE_STATE), power_manager_get_state_string(new_state), pm_seconds);
        pm_state = new_state;
        pm_old_state = new_state;
        pm_seconds = 0;

    }
}

unsigned char power_manager_get_state(void)
{
    return pm_state;
}

const rom char* power_manager_get_state_string(unsigned char state)
{

    switch(state)
    {
        case PM_STARTUP:
            return tr(S_PM_STARTUP);
            break;

        case PM_STARTUP_SWR_LIMIT:
            return tr(S_PM_STARTUP_SWR_LIMIT);
            break;

        case PM_STARTUP_TIMED_LIMIT:
            return tr(S_PM_STARTUP_TIMED_LIMIT);
            break;

        case PM_STARTUP_DELTA_LIMIT:
            return tr(S_PM_STARTUP_DELTA_LIMIT);
            break;

        case PM_STARTUP_SWR_BOOST:
            return tr(S_PM_STARTUP_SWR_BOOST);
            break;

        case PM_STARTUP_TIMED_BOOST:
            return tr(S_PM_STARTUP_TIMED_BOOST);
            break;

        case PM_STARTUP_DELTA_BOOST:
            return tr(S_PM_STARTUP_DELTA_BOOST);
            break;

        case PM_HEATING:
            return tr(S_PM_HEATING);
            break;

        case PM_HEATED:
            return tr(S_PM_HEATED);
            break;

        case PM_IDLE:
            return tr(S_PM_IDLE);
            break;

        case PM_ENTER_IDLE:
            return tr(S_PM_ENTER_IDLE);
            break;

        case PM_ENTER_SLEEP:
            return tr(S_PM_ENTER_SLEEP);
            break;

        case PM_SLEEP:
            return tr(S_PM_SLEEP);
            break;

        case PM_WAKEUP:
            return tr(S_PM_WAKEUP);
            break;

        case PM_WAKEUP_SWR_BOOST:
            return tr(S_PM_WAKEUP_SWR_BOOST);
            break;

        case PM_WAKEUP_TIMED_BOOST:
            return tr(S_PM_WAKEUP_TIMED_BOOST);
            break;

        case PM_WAKEUP_DELTA_BOOST:
            return tr(S_PM_WAKEUP_DELTA_BOOST);
            break;

        case PM_ENTER_POWERDOWN:
            return tr(S_PM_ENTER_POWERDOWN);
            break;

        case PM_POWERDOWN:
            return tr(S_PM_POWERDOWN);
            break;

        case PM_MANUAL_OFF:
            return tr(S_PM_MANUAL_OFF);
            break;

        case PM_MANUAL_LIMIT:
            return tr(S_PM_MANUAL_LIMIT);
            break;

        case PM_MANUAL_NORMAL:
            return tr(S_PM_MANUAL_NORMAL);
            break;

        case PM_MANUAL_BOOST:
            return tr(S_PM_MANUAL_BOOST);
            break;

        case PM_OFF:
            return tr(S_PM_OFF);
            break;

        default:
            return tr(S_INVALID);
            break;


    }
}

void power_manager_set_config(unsigned char cfg)
{
    config.pm_config = cfg;
}

void power_manager_set_sleeptime(unsigned short stime)
{
    config.sleep_time = stime;
}

void power_manager_ticker(void)
{
    adc_access_mutex |= ADC_MUTEX_DELTACALC;
    adc_access_mutex &= ~ADC_MUTEX_DELTACALC;
}

unsigned long power_manager_get_state_seconds(void)
{
    return pm_seconds;
}
