/*
 * (c) 2013 Ch. klippel  -  ck@atelier-klippel.de / c.klippel@gmx.de
 *
 * This code is released under the terms of the GPL, version 2 or newer.
 * See the file COPYING for more information about the license.
 *
 * Changelog:
 *
 * v00.01: - Initial version
 *
 */

#include "rfsupply.h"

static unsigned short total_delta, stored_delta, old_delta, delta_diff;
static unsigned short old_vf, old_vr, old_swr, old_supply;
static unsigned char mode;

void rf_manager_init(void)
{
    uart_print_start_msg(tr(S_RF_MANAGER));

    adc_access_mutex |= ADC_MUTEX_DELTACALC;
    total_delta = 0;
    stored_delta = 0;
    old_delta = 0;
    delta_diff = 0;
    old_vf = 0;
    old_vr = 0;
    old_swr = 0;
    old_supply = 0;
    mode = 0;
    adc_access_mutex &= ~ADC_MUTEX_DELTACALC;
}

void rf_manager_add_delta(void)
{
    total_delta += get_delta_ushort(rf_data.supply, old_supply);
    total_delta += get_delta_ushort(rf_data.vf, old_vf);
    total_delta += get_delta_ushort(rf_data.vr, old_vr);
    total_delta += get_delta_ushort(rf_data.swr, old_swr);
    
    if(total_delta >= 10000)
    {
        total_delta = 9999;
    }

    old_supply = rf_data.supply;
    old_vf = rf_data.vf;
    old_vr = rf_data.vr;
    old_swr = rf_data.swr;
}

void rf_manager_store_delta(void)
{
    adc_access_mutex |= ADC_MUTEX_DELTACALC;
    old_delta = stored_delta;
    stored_delta = total_delta;
    delta_diff = get_delta_ushort(stored_delta, old_delta);
    adc_access_mutex &= ~ADC_MUTEX_DELTACALC;
}

void rf_manager_clear_delta(void)
{
    adc_access_mutex |= ADC_MUTEX_DELTACALC;
    total_delta = 0;
    adc_access_mutex &= ~ADC_MUTEX_DELTACALC;
}

void rf_manager_clear_all_delta(void)
{
    adc_access_mutex |= ADC_MUTEX_DELTACALC;
    total_delta = 0;
    stored_delta = 0;
    old_delta = 0;
    delta_diff = 0;
    adc_access_mutex &= ~ADC_MUTEX_DELTACALC;
}

unsigned short rf_manager_get_delta(void)
{
    return stored_delta;
}

unsigned short rf_manager_get_delta_diff(void)
{
    return delta_diff;
}

unsigned char rf_manager_has_delta(unsigned short delta)
{
    adc_access_mutex |= ADC_MUTEX_DELTACALC;
    if(total_delta >= delta)
    {
        adc_access_mutex &= ~ADC_MUTEX_DELTACALC;
        return 1;
    }
    else
    {
        adc_access_mutex &= ~ADC_MUTEX_DELTACALC;
        return 0;
    }
}

unsigned char rf_manager_has_delta_diff(unsigned short delta)
{
    if(delta_diff >= delta)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

unsigned short rf_manager_get_swr(void)
{
    static unsigned short swr;

    adc_access_mutex |= ADC_MUTEX_DELTACALC;
    swr = rf_data.swr;
    adc_access_mutex &= ~ADC_MUTEX_DELTACALC;

    return swr;
}

unsigned short rf_manager_get_pwr(unsigned char scale)
{
    static float pwr, sc;
    static unsigned short res;

    sc = scale;

    adc_access_mutex |= ADC_MUTEX_DELTACALC;
    pwr = rf_data.pwr;
    adc_access_mutex &= ~ADC_MUTEX_DELTACALC;

    pwr *= scale;
    res = pwr;
    
    return res;

}

unsigned short rf_manager_get_supply(void)
{
    static unsigned short supply;

    adc_access_mutex |= ADC_MUTEX_DELTACALC;
    supply = rf_data.supply;
    adc_access_mutex &= ~ADC_MUTEX_DELTACALC;

    return supply;
}

void rf_manager_ticker(void)
{
    adc_access_mutex |= ADC_MUTEX_DELTACALC;
    old_delta = stored_delta;
    stored_delta = total_delta;
    total_delta = 0;
    adc_access_mutex &= ~ADC_MUTEX_DELTACALC;
}

void rf_manager_set_mode(unsigned char val)
{
    if(val != mode)
    {

//        uart_print_timed_change_msg(tr(S_RF_MANAGER), tr(S_CHANGE_POWERLEVEL), rf_manager_get_mode_string(val), power_manager_get_state_seconds());
        uart_print_module_msg_val(tr(S_RF_MANAGER), tr(S_CHANGE_POWERLEVEL), rf_manager_get_mode_string(val));
        mode = val;

        if(val == RF_POWER_OFF)
        {
            RFG_EN = 0;
            VLIM_DIR = PIN_IN;
        }
        else if(val == RF_POWER_NORMAL)
        {
            VLIM_DIR = PIN_IN;
            RFG_EN = 1;
        }
        else if(val == RF_POWER_BOOST)
        {
            VLIM_OUT = 0;
            VLIM_DIR = PIN_OUT;
            RFG_EN = 1;
        }
        else if(val == RF_POWER_LIMIT)
        {
            VLIM_OUT = 1;
            VLIM_DIR = PIN_OUT;
            RFG_EN = 1;
        }
    }
}

unsigned char rf_manager_get_mode(void)
{
    return mode;
}

const rom char* rf_manager_get_mode_string(unsigned char val)
{
    switch(val)
    {
        case RF_POWER_OFF:
            return tr(S_OFF);
            break;

        case RF_POWER_NORMAL:
            return tr(S_NORMAL);
            break;

        case RF_POWER_BOOST:
            return tr(S_BOOST);
            break;

        case RF_POWER_LIMIT:
            return tr(S_LIMIT);
            break;

        default:
            return tr(S_INVALID);
            break;
    }
}
