/*
 * (c) 2013 Ch. klippel  -  ck@atelier-klippel.de / c.klippel@gmx.de
 *
 * This code is released under the terms of the GPL, version 2 or newer.
 * See the file COPYING for more information about the license.
 *
 * Changelog:
 *
 * v00.01: - Initial version
 *
 */

#include "rfsupply.h"

#pragma udata access _TIP_DETECT_DATA
static near unsigned char tip_status;
static near unsigned short ticker_cnt;
near volatile unsigned char ticker;
near volatile unsigned char fticker;
#pragma udata
static unsigned char old_tip_status;

void tip_detect_init(void)
{
    uart_print_start_msg(tr(S_TIP_DETECTOR));
    
    ticker = 0;
    fticker = 0xFF;
    ticker_cnt = 0;
    
    IPR1bits.TMR1IP = 1;
    PIR1bits.TMR1IF = 0;

    if(TIP_DET == 1)
    {
        tip_status = TIP_OK;
        old_tip_status = TIP_OK;
    }
    else
    {
        tip_status = TIP_MISSING;
        old_tip_status = TIP_MISSING;
    }

    T1CON = 0b00110000;

    PIE1bits.TMR1IE = 1;

    T1CONbits.TMR1ON = 1;
}

#pragma interrupt tip_detect_process
void tip_detect_process(void)
{
    if(RFG_EN)
    {
        if(TIP_DET == 0)
        {
            tip_status = TIP_MISSING;
        }
        else
        {
            tip_status = TIP_OK;
        }
    }
    
    ticker_cnt++;

    if(ticker_cnt >= 3906)
    {
        ticker_cnt = 0;
        ticker++;
    }
    if(fticker)
    {
        fticker--;
    }
    PIR1bits.TMR1IF = 0;
}

#pragma code

unsigned char tip_detect_get_status(void)
{
    if(old_tip_status != tip_status)
    {
        old_tip_status = tip_status;

        if(tip_status == TIP_OK)
        {
            uart_print_module_msg(tr(S_TIP_DETECTOR), tr(S_TIP_DETECTED));
        }
        else
        {
            uart_print_module_msg(tr(S_TIP_DETECTOR), tr(S_NO_TIP));
        }
    }
    return tip_status;
}
