/*
 * (c) 2013 Ch. klippel  -  ck@atelier-klippel.de / c.klippel@gmx.de
 *
 * This code is released under the terms of the GPL, version 2 or newer.
 * See the file COPYING for more information about the license.
 *
 * Changelog:
 *
 * v00.01: - Initial version
 *
 */

#include "rfsupply.h"

void uart_init(void)
{
    unsigned short delay;
    
    uart_fifo_init();
    
    TXSTA = 0b00100100;
    RCSTA = 0b10010000;

    //	BAUDCONbits.BRG16 = 1;
    //	SPBRG = 136; // 57600 baud

    BAUDCONbits.BRG16 = 0;
    SPBRG = 51; // 38400 baud
    //	SPBRG = 25; // 38400 baud


    IPR1bits.RCIP = 0;
    PIR1bits.RCIF = 0;
    RCSTAbits.CREN = 0;
    RCSTAbits.CREN = 1;
    PIE1bits.RCIE = 1;
    IPR1bits.TXIP = 0;
    PIE1bits.TXIE = 0;

    for(delay = 0; delay < 20000; delay++)
    {
        _asm NOP _endasm
    }

    uart_print_romstring("\r\n\r\n");
    uart_print_version();
}

void uart_print_romstring(const rom char *str)
{
    while(*str)
    {
        uart_fifo_putchar((unsigned char)*str++);
    }
}

void uart_print_ramstring(char *str)
{
    while(*str)
    {
        uart_fifo_putchar((unsigned char)*str++);
    }
}

void uart_print_start_msg(const rom char *str)
{
    uart_print_romstring(tr(S_STARTING));
    uart_print_romstring(": ");
    uart_print_romstring(str);
    uart_print_romstring("\r\n");
}

void uart_print_module_msg(const rom char *mod_str, const rom char *msg_str)
{
    uart_print_romstring(mod_str);
    uart_print_romstring(": ");
    uart_print_romstring(msg_str);
    uart_print_romstring("\r\n");
}

void uart_print_module_msg_val(const rom char *mod_str, const rom char *msg_str, const rom char *val_str)
{
    uart_print_romstring(mod_str);
    uart_print_romstring(": ");
    uart_print_romstring(msg_str);
    uart_print_romstring(": \"");
    uart_print_romstring(val_str);
    uart_print_romstring("\"\r\n");
}

void uart_print_module_msg_ramval(const rom char *mod_str, const rom char *msg_str, char *val_str)
{
    uart_print_romstring(mod_str);
    uart_print_romstring(": ");
    uart_print_romstring(msg_str);
    uart_print_romstring(": \"");
    uart_print_ramstring(val_str);
    uart_print_romstring("\"\r\n");
}

void uart_print_timed_change_msg(const rom char *mod_str, const rom char *msg_str, const rom char *state_str, unsigned long secs)
{
    uart_print_romstring(mod_str);
    uart_print_romstring(": ");
    uart_print_romstring(tr(S_AFTER));
    uart_print_romstring(" ");
    uart_print_seconds_as_time(secs);
    uart_print_romstring(" -> ");
    uart_print_romstring(msg_str);
    uart_print_romstring(" \"");
    uart_print_romstring(state_str);
    uart_print_romstring("\"\r\n");
}

void uart_print_ushortnum(unsigned short num, unsigned char digits)
{
    divmod_set_remainder(num);

    if(digits == 5)
    {
        goto FIVE_DIGITS;
    }
    else if(digits == 4)
    {
        goto FOUR_DIGITS;
    }
    else if(digits == 3)
    {
        goto THREE_DIGITS;
    }
    else if(digits == 2)
    {
        goto TWO_DIGITS;
    }
    else if(digits == 1)
    {
        goto ONE_DIGIT;
    }

FIVE_DIGITS:
    divmod_set_divider(10000);
    uart_fifo_putchar('0'+divmod());

FOUR_DIGITS:
    divmod_set_divider(1000);
    uart_fifo_putchar('0'+divmod());

THREE_DIGITS:
    divmod_set_divider(100);
    uart_fifo_putchar('0'+divmod());

TWO_DIGITS:
    divmod_set_divider(10);
    uart_fifo_putchar('0'+divmod());

ONE_DIGIT:
    uart_fifo_putchar('0'+divmod_get_remainder());
}

void uart_print_seconds_as_time(unsigned short seconds)
{
    seconds_to_time(seconds);
    uart_print_ushortnum(time_data.hours, 3);
    uart_fifo_putchar(':');
    uart_print_ushortnum(time_data.minutes, 2);
    uart_fifo_putchar(':');
    uart_print_ushortnum(time_data.seconds, 2);
}

void uart_print_version(void)
{
    uart_print_romstring("<<< ");
    uart_print_romstring(PROJECT_NAME);
    uart_print_romstring(" Version ");
    uart_print_ushortnum(VERSION_MAJOR, 2);
    uart_print_romstring(".");
    uart_print_ushortnum(VERSION_MINOR, 2);
    uart_print_romstring(" >>>\r\n");
}
